#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
用于Q-learning的学习,内含MDP环境和Q-learning决策大脑
代码是根据莫凡Q-leaning教学视频修改而来
在这感谢莫凡的工作
附莫凡Q-learing学习网址：https://mofanpy.com/tutorials/machine-learning/reinforcement-learning/intro-q-learning/
我的GitHub地址：https://github.com/353055619/Q_learning_utils.git
"""

__author__ = 'Godw'

__all__ = ['env','Q_learning']
